@@ -1,4 +1,4 @@
-# $Id: Lambda.pm,v 1.48 2008/05/30 11:44:27 dk Exp $
+# $Id: Lambda.pm,v 1.53 2008/07/31 14:48:04 dk Exp $
 
 package IO::Lambda;
 
@@ -14,7 +14,7 @@ use vars qw(
 	$THIS @CONTEXT $METHOD $CALLBACK
 	$DEBUG
 );
-$VERSION     = '0.20';
+$VERSION     = '0.21';
 @ISA         = qw(Exporter);
 @EXPORT_CONSTANTS = qw(
 	IO_READ IO_WRITE IO_EXCEPTION 
@@ -26,7 +26,7 @@ $VERSION     = '0.20';
 );
 @EXPORT_LAMBDA = qw(
 	this context lambda this_frame again
-	io read write sleep tail tails tailo state
+	io read write readwrite sleep tail tails tailo state
 );
 @EXPORT_OK   = (@EXPORT_LAMBDA, @EXPORT_CONSTANTS, @EXPORT_STREAM);
 %EXPORT_TAGS = (
@@ -345,7 +345,7 @@ sub lambda_handler
 # The object becomes stopped, so no new events will be allowed to register.
 sub cancel_all_events
 {
-	my $self = shift;
+	my ( $self, %opt) = @_;
 
 	$self-> {stopped} = 1;
 
@@ -357,9 +357,19 @@ sub cancel_all_events
 	@{$self-> {in}} = (); 
 
 	if ( $arr) {
+		my $cascade = $opt{cascade};
+		my (%called, @cancel);
 		for my $rec ( @$arr) {
 			next unless my $watcher = $rec-> [WATCH_OBJ];
+			# global destruction in action! this should be $self, but isn't
+			next unless ref($rec-> [WATCH_LAMBDA]); 
 			$watcher-> lambda_handler( $rec);
+			push @cancel, $watcher if $cascade;
+		}
+		for ( @cancel) {
+			next if $called{"$_"};
+			$called{"$_"}++;
+			$_-> cancel_all_events(%opt);
 		}
 	}
 }
@@ -436,6 +446,12 @@ sub terminate
 	warn $self-> _msg('terminate') if $DEBUG;
 }
 
+# propagate event destruction on all levels
+sub destroy
+{
+	shift-> cancel_all_events( cascade => 1);
+}
+
 # synchronisation
 
 # drives objects dependant on the other objects until all of them
@@ -552,6 +568,8 @@ sub lambda(&)
 	$l;
 }
 
+*io = \&lambda;
+
 # re-enter the latest (or other) frame
 sub again
 {
@@ -592,14 +610,14 @@ sub add_watch
 	)
 }
 
-# io($flags,$handle,$deadline)
-sub io(&)
+# readwrite($flags,$handle,$deadline)
+sub readwrite(&)
 {
-	return $THIS-> override_handler('io', \&io, shift)
-		if $THIS-> {override}->{io};
+	return $THIS-> override_handler('readwrite', \&io, shift)
+		if $THIS-> {override}->{readwrite};
 
 	$THIS-> add_watch( 
-		shift, \&io,
+		shift, \&readwrite,
 		@CONTEXT[0,1,2,0,1,2]
 	)
 }
@@ -852,8 +870,10 @@ sub getline
 	lambda {
 		my ( $fh, $buf, $deadline) = @_;
 		context readbuf($reader), $fh, $buf, qr/^[^\n]*\n/, $deadline;
-		&tail();
-	}
+	tail {
+		substr( $$buf, 0, length($_[0]), '') unless defined $_[1];
+		@_;
+	}}
 }
 
 # write whole buffer to stream
@@ -970,49 +990,63 @@ programming with single-process, single-thread, non-blocking I/O.
 
 =head1 SYNOPSIS
 
-=head2 Basics
+This is a fairly large document, so depending on your reading tastes, you may
+either read all from here - it begins with code examples, then with more code
+examples, then the explanation of basic concepts, and finally gets to the
+complex ones. Or, you may skip directly to the fun part (L<Stream IO>, where
+functional style mixes with I/O. Also note that C<io> and C<lambda> are synonyms.
 
-Prerequisite
+=head2 Read line by line from filehandle
 
-    use IO::Lambda qw(:lambda);
+Given C<$filehandle> is non-blocking, the following code creates a lambda than
+reads from it util EOF or error occured. C<getline> (see L<Stream IO> below) is
+a similar lambda that reads single line from a filehandle.
 
-Create an empty IO::Lambda object
+    use IO::Lambda qw(:all);
 
-    my $q = lambda {};
+    sub my_reader
+    {
+       my $filehandle = shift;
+       io {
+           context getline, $filehandle, \(my $buf = '');
+       tail {
+           my ( $string, $error) = @_;
+           if ( $error) {
+               warn "error: $error\n";
+           } else {
+               print $string;
+               return again;
+           }
+       }}
+    }
 
-Wait for it to finish
+Assume we have two socket connections, and sockets are non-blocking - read from
+both of them simulteously. The following code creates a lambda that reads from
+two readers:
 
-    $q-> wait;
+    sub my_reader_all
+    {
+        my @filehandles = @_;
+	lambda {
+	    context map { my_reader($_) } @filehandles;
+	    tails { print "all is finished\n" };
+	}
+    }
 
-Create lambda object and get its value
+    my_reader_all( $socket1, $socket2)-> wait;
 
-    $q = lambda { 42 };
-    print $q-> wait; # will print 42
-
-Create pipeline of two lambda objects
-
-    $q = lambda {
-        context lambda { 42 };
-	tail { 1 + shift };
-    };
-    print $q-> wait; # will print 43
-
-Create pipeline that waits for 2 lambdas
-
-    $q = lambda {
-        context lambda { 2 }, lambda { 3 };
-	tails { sort @_ }; # order is not guaranteed
-    };
-    print $q-> wait; # will print 23
-
-=head2 Non-blocking I/O
+=head2 Non-blocking HTTP
 
 Given a socket, create a lambda that implements http protocol
 
+    use IO::Lambda qw(:all);
+    use IO::Socket;
+    use HTTP::Request;
+
     sub talk
     {
         my $req    = shift;
-        my $socket = IO::Socket::INET-> new( PeerAddr => $req-> host, PeerPort => 80);
+        my $socket = IO::Socket::INET-> new( PeerAddr => 'www.perl.com', PeerPort => 80);
 
 	lambda {
 	    context $socket;
@@ -1028,6 +1062,7 @@ Given a socket, create a lambda that implements http protocol
 	}
     }
 
+
 Connect and talk to the remote
 
     $request = HTTP::Request-> new( GET => 'http://www.perl.com');
@@ -1267,7 +1302,7 @@ many bytes from a socket within 5 seconds:
 
 Rewriting the same code with C<read> semantics that accepts time as timeout instead, 
 would be not that elegant:
-   
+
    lambda {
        my $buf = '';
        my $time_left = 5;
@@ -1328,6 +1363,10 @@ or by using the package syntax,
 
 Creates a new C<IO::Lambda> object.
 
+=item io()
+
+Same as C<lambda>.
+
 =item read($filehandle, $deadline = undef)
 
 Executes either when C<$filehandle> becomes readable, or after C<$deadline>.
@@ -1339,7 +1378,7 @@ that means that it will never be called with FALSE.
 
 Exactly same as C<read>, but executes when C<$filehandle> becomes writable.
 
-=item io($flags, $filehandle, $deadline = undef)
+=item readwrite($flags, $filehandle, $deadline = undef)
 
 Executes either when C<$filehandle> satisfies any of the condition C<$flags>,
 or after C<$deadline>. C<$flags> is a combination of three integer constants,
@@ -1448,7 +1487,7 @@ a kind of restartable continuations.
 
 =back
 
-=head2 Stream I/O
+=head2 Stream IO
 
 The whole point of this module is to help building complex protocols in a
 clear, consequent programming style. Consider how perl's low-level C<sysread>
@@ -1632,6 +1671,13 @@ Cancels all watchers and resets lambda to the stopped state.  If there are any
 lambdas that watch for this object, these will be notified first. C<@args> will
 be stored and available for later calls by C<peek>.
 
+=item destroy
+
+Cancels all watchers and resets lambda to the stopped state. Does the same to
+all lambdas the caller lambda watches after, recursively. Useful where
+explicit, long-lived lambdas shouldn't be subject to global destruction, which
+kills objects in random order; C<destroy> kills them in some order, at least.
+
 =item wait @args
 
 Waits for the caller lambda to finish, returns the result of C<peek>.
@@ -1726,7 +1772,7 @@ of a certain type; for example the code
    }}
 
 is therefore better to be written as
-    
+
    state A => tail {
    state B => tail {
       ...
@@ -1758,6 +1804,24 @@ L<IO::Lambda::HTTP::Authen::NTLM> requires L<Authen::NTLM>.
 
 =back
 
+=head1 BENCHMARKS
+
+=over
+
+=item *
+
+Single-process tcp client and server; server echoes back everything is sent by
+the client. 500 connections sequentially created, instructed to send a single
+line to the server, and destroyed.
+
+                        2.4GHz x86-64 linux 1.2GHz win32
+  Lambda using select       0.694 sec        6.364 sec
+  Lambda using AnyEvent     0.684 sec        7.031 sec
+  Raw sockets using select  0.145 sec        4.141 sec
+  POE using select          5.349 sec       14.887 sec
+
+=back
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright (c) 2008 capmon ApS. All rights reserved.
